/*

Copyright (c) 2004, Roland J. Weiss.
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, this
    list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice,
    this list of conditions and the following disclaimer in the documentation
    and/or other materials provided with the distribution.
  * Neither the name of the University of Tuebingen nor the names of its
    contributors may be used to endorse or promote products derived from this
    software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*/

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

class PDwrapHandler extends DefaultHandler
{
  PDwrapHandler()
  {
    currentElement = noElem;
    prolog = new String();
    epilog = new String();
  }

  // Behavior:
  // prolog/epilog elements -> add characters to map with escapes handled
  
  // Method override.
  // Depending on the element name, set the active element.
  public void startElement(String namespaceURI,
                           String localName,
                           String qName,
                           Attributes atts) throws SAXException
  {
    //System.out.println("[start] qName: " + qName);
    if (qName.equals(prologStr))
    {
      currentElement = prologElem;
    }
    else if (qName.equals(epilogStr))
    {
      currentElement = epilogElem;
    }
    else
    {
      currentElement = noElem;
    }
    //for (int i = 0; i < atts.getLength(); ++i)
    //{
    //  System.out.println(" attribute " + atts.getQName(i) + ": " + atts.getValue(i));
    //}
  }
  
  // Method override.
  // Depending on the element name, expand the strings that were collected in
  // the characters method.
  public void endElement(String namespaceURI,
                         String localName,
                         String qName) throws SAXException
  {
    //System.out.println("[end] qName: " + qName);
    if (qName.equals(prologStr))
    {
      prolog = StringExpander.expand(prolog);
      //System.out.print("<prolog>\n" + prolog);
      //System.out.println("</prolog>");
    }
    else if (qName.equals(epilogStr))
    {
      epilog = StringExpander.expand(epilog);
      //System.out.print("<epilog>\n" + epilog);
      //System.out.println("</epilog>");
    }
    currentElement = noElem;
  }

  // Method override.
  // Depending on the active elements, append the extracted characters to the
  // correct string.
  public void characters(char[] ch,
                         int start,
                         int length) throws SAXException
  {
    if (currentElement != noElem)
    {
      char[] chars = new char[length];
      for (int i = 0; i < length; ++i) { chars[i] = ch[i+start]; }
      String cs = new String(chars);
      //System.out.println(" characters length: " + length);
      //System.out.println(" characters: " + cs);
      //System.out.println(" expanded: " + StringExpander.expand(cs));

      if (currentElement == prologElem) prolog += cs;
      else if (currentElement == epilogElem) epilog += cs;
    }
  }

  // Reader methods for the prolog and epilog strings.
  public String getProlog() { return prolog; }
  public String getEpilog() { return epilog; }

  // DATA

  // Strings that hold the expanded prolog and epilog sections after processing.
  String prolog;
  String epilog;
  
  // Variable that denotes the active element.
  int currentElement;

  // Integer constants that denote the possible elements.
  final int noElem = 0;
  final int prologElem = 1;
  final int epilogElem = 2;

  // String constants for prolog and epilog element names.
  final String prologStr = "prolog";
  final String epilogStr = "epilog";
}
